#+TITLE: JISON Source Code Blocks in Org Mode
#+AUTHOR: Peter Moresi

* Introductions

  [[http://zaach.github.io/jison/][JISON]] is a port of GNU Bison to JavaScript. 

  This tool is a compiler generator that allows you to extend the browser and other JavaScript execution environments with new languages.

  [[http://zaach.github.io/jison/assets/images/jison_x1.png]]

* Requirement & Setup

  JISON is distributed as an open source project with a permissive MIT-like license.

  You can configure Org mode to execute jison source code blocks by adding a line to org-babel-load-languages:

  #+BEGIN_SRC emacs-lisp
    (org-babel-do-load-languages
     'org-babel-load-languages
     '((jison . t))) ; this line activates jison
  #+END_SRC

  If you desire syntax highlighting then you can use [[https://github.com/petermoresi/bison-mode][bison-mode]].

* Org Mode Features for jison Source Code Blocks

  JISON source code blocks produce a bottom-up parser in JavaScript. The default value for the :results header argument is "file" and for the :exports header argument it is "results".

  There are two jison specific header arguments that can be used to tailor the command line. They are:

  1. :cmd
     - this header argument can be used to change the program from the default "jison".
  2. :cmdline
     - the default value is "". A sensible values is "-m js". See jison --help for more options.

  The :file header argument is required for jison source code blocks.

* Sessions

  JISON does not support sessions.

* Result Types

  JISON source code blocks produce JavaScript files, so the default value "file" is the only sensible type of result.

* Examples of Use
** Create a Parser for your Language

  A typical use of jison is to create your own calculator language.

  #+NAME: calculator
  #+BEGIN_SRC jison :file calculator.js :exports both

    /* description: Parses and executes mathematical expressions. */

    /* lexical grammar */
    %lex
    %%

    \s+                   /* skip whitespace */
    [0-9]+("."[0-9]+)?\b  return 'NUMBER'
    "*"                   return '*'
    "/"                   return '/'
    "-"                   return '-'
    "+"                   return '+'
    "^"                   return '^'
    "!"                   return '!'
    "%"                   return '%'
    "("                   return '('
    ")"                   return ')'
    "PI"                  return 'PI'
    "E"                   return 'E'
    <<EOF>>               return 'EOF'
    .                     return 'INVALID'

    /lex

    /* operator associations and precedence */

    %left '+' '-'
    %left '*' '/'
    %left '^'
    %right '!'
    %right '%'
    %left UMINUS

    %start expressions

    %% /* language grammar */

    expressions
        : e EOF
            { return $1; }
        ;

    e
        : e '+' e
            {$$ = $1+$3;}
        | e '-' e
            {$$ = $1-$3;}
        | e '*' e
            {$$ = $1*$3;}
        | e '/' e
            {$$ = $1/$3;}
        | e '^' e
            {$$ = Math.pow($1, $3);}
        | e '!'
            {{
              $$ = (function fact (n) { return n==0 ? 1 : fact(n-1) * n })($1);
            }}
        | e '%'
            {$$ = $1/100;}
        | '-' e %prec UMINUS
            {$$ = -$2;}
        | '(' e ')'
            {$$ = $2;}
        | NUMBER
            {$$ = Number(yytext);}
        | E
            {$$ = Math.E;}
        | PI
            {$$ = Math.PI;}
        ;

  #+END_SRC

  Executing the source code block above will result in a JavaScript file.

  #+RESULTS: calculator
  [[file:calculator.js]]

** Run your new language

   This source code block loads the calculator and parses a mathmatically expression. 

  #+BEGIN_SRC js :var folder=(file-name-directory buffer-file-name)
    var calculator = require(folder + "calculator.js")

    return calculator.parse("5*PI^2");
  #+END_SRC

  #+RESULTS:
  : 49.34802200544679



   
